;-----------------------------------------------------------------------------
; File: Meshes.cpp
;
; Desc: For advanced geometry, most apps will prefer to load pre-authored
;       meshes from a file. Fortunately, when using meshes, D3DX does most of
;       the work for this, parsing a geometry file and creating vertx buffers
;       (and index buffers) for us. This tutorial shows how to use a D3DXMESH
;       object, including loading it from a file and rendering it. One thing
;       D3DX does not handle for us is the materials and textures for a mesh,
;       so note that we have to handle those manually.
;
;       Note: one advanced (but nice) feature that we don't show here is that
;       when cloning a mesh we can specify the FVF. So, regardless of how the
;       mesh was authored, we can add/remove normals, add more texture
;       coordinate sets (for multi-texturing), etc.
;
;-----------------------------------------------------------------------------
;################################################################
InitD3D PROC  hWnd:DWORD 
			Local  d3dpp:D3DPRESENT_PARAMETERS
			Local  retour:DWORD
			ZEROLOCALES retour	
			
		invoke Direct3DCreate9,D3D_SDK_VERSION
		.if eax != 0
			mov ppvIDirect3D9,eax
			mov retour,eax
			mov d3dpp.Windowed,TRUE
			mov d3dpp.SwapEffect,D3DSWAPEFFECT_DISCARD
			mov d3dpp.BackBufferFormat,D3DFMT_UNKNOWN
		    	mov d3dpp.EnableAutoDepthStencil, TRUE		;idem que pour lights et textures
			mov d3dpp.AutoDepthStencilFormat,D3DFMT_D16	;		
			;InitD3D ne fonctionne pas dans WM_CREATE
			IDirect3D9 CreateDevice,D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL, hWnd,\
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,\
                                      addr d3dpp, addr ppvIDirect3DDevice9
			.if eax == D3D_OK		;0
    				;Turn on ambient lighting 
    				IDirect3DDevice9  SetRenderState,D3DRS_AMBIENT, 0ffffffffh
				; Turn on the zbuffer ++++++
				IDirect3DDevice9  SetRenderState,D3DRS_ZENABLE, TRUE
		
				mov retour,1	;sinon retour = 0
				;la fonction fournit le pointeur sur l'interface IDirect3DDevice9
			.endif 
			;c'est fait ,on peut maintenant visualiser la fentre 
			;la fentre D3D doit tre rafraichit de manire particulire dans WM_PAINT			
			
		.endif
 FindeInitD3D:
         mov eax,retour
         ret
InitD3D endp
;tout de suite aprs avoir crer le device avec InitD3D
;################################################################

InitVB PROC uses esi edi ebx
			Local CNT:DWORD,index,ppvID3DXBuffer,decalage
			Local pd3dxMaterials:DWORD
			Local theta:REAL4
			Local PasseQword:REAL8
			Local  retour:DWORD
			ZEROLOCALES retour
	;ID3DXBaseMesh provides the fundamentals. ID3DXMesh and ID3DXPMesh 
	;inherit from ID3DXBaseMesh pDevice,ppAdjacency,ppMaterials,ppEffectInstances,
	;pNumMaterials,ppMesh	
	mov retour,0
	cld
	;D3DXLoadMeshFromX @"#!;& "Couldn't find .x"
	invoke D3DXLoadMeshFromXResource,NULL,addr tigerres,addr meshesTypeRes,\
			D3DXMESH_SYSTEMMEM,ppvIDirect3DDevice9,NULL,\
			addr ppvID3DXBuffer,NULL,addr dwNumMaterials,addr ppvID3DXMesh 
	; We need to extract the material properties and texture names from the 
	; ppvID3DXBuffer
	ID3DXBuffer GetBufferPointer	
	mov pd3dxMaterials,eax 		;p sur D3DXMATERIAL structure
	;------------------------- rserv la mmoire com -------------------------
	invoke CoGetMalloc,1,addr ppvIMalloc
	xor edx,edx
	mov ecx,dwNumMaterials
	mov eax, sizeof D3DMATERIAL9
	mul ecx
	IMalloc Alloc,eax			;similaire a malloc du c
	mov g_pMeshMaterials,eax
	xor edx,edx
	mov ecx,dwNumMaterials
	mov eax, sizeof DWORD		;p sur DIRECT3DTEXTURE9
	mul ecx
	IMalloc Alloc,eax			;similaire a malloc du c	Free
	mov g_pMeshTextures,eax
	;------ init memory 0 ---------------
	mov edi,g_pMeshTextures
	mov eax,0
	mov ecx,dwNumMaterials
	rep stosd	
	;---------------- recopier et modifier -------------------------------------
	mov index,0
	PuPo CNT,dwNumMaterials
	;pd3dxMaterials pointeur sur D3DXMATERIAL * dwNumMaterials,crer par D3DXLoadMeshFromXResource
	;g_pMeshMaterials array of D3DMATERIAL9
	
	;D3DXMATERIAL.D3DMATERIAL9 = MatD3D
	;D3DXMATERIAL	STRUCT		;d3dx9mesh.inc
	;	MatD3D D3DMATERIAL9 <> 
	;	pTextureFilename DWORD ?
	;D3DXMATERIAL		ENDS	
	; D3DMATERIAL9	STRUCT		;d3d9types.inc
	;	Diffuse D3DCOLORVALUE <>   ; Diffuse color RGBA 
	;	Ambient D3DCOLORVALUE <>   ; Ambient color RGB 
	;	Specular D3DCOLORVALUE <>   ; Specular 'shininess' 
	;	Emissive D3DCOLORVALUE <>   ; Emissive color RGB 
	;	Power DWORD ?  ; Sharpness if specular highlight 
	;D3DMATERIAL9		ENDS
	;D3DCOLORVALUE	STRUCT
	;	r DWORD ?
	;	g DWORD ?
	;	b DWORD ?
	;	a DWORD ?
	;D3DCOLORVALUE	ENDS	

	.while(CNT)
		; 	
		; Copy the material
		;g_pMeshMaterials[i] = pd3dxMaterials[i].MatD3D;
		mov esi,index
		imul esi,sizeof D3DXMATERIAL
		add esi,pd3dxMaterials		
		lea esi,(D3DXMATERIAL ptr [esi]).MatD3D ;MatD3D = D3DMATERIAL9
		mov edi,index
		imul edi,sizeof D3DMATERIAL9
		add edi,g_pMeshMaterials
		mov ecx,sizeof D3DMATERIAL9
		rep movsb		
		; Set the ambient color for the material (D3DX does not do this)
		;g_pMeshMaterials[i].Ambient = g_pMeshMaterials[i].Diffuse;
		mov edi,index
		imul edi,sizeof D3DMATERIAL9
		add edi,g_pMeshMaterials
		mov esi,edi
		lea edi,(D3DMATERIAL9 ptr [edi]).Ambient
		lea esi,(D3DMATERIAL9 ptr [esi]).Diffuse
		mov ecx,sizeof D3DMATERIAL9.Diffuse	;DWORD
		rep movsb
		;--------------------- TEXTURES ----------------------------
		mov esi,index
		imul esi,sizeof D3DXMATERIAL	;72	; 00000048H
		.LISTALL
		add esi,pd3dxMaterials		
		.NOLIST		
		lea esi,(D3DXMATERIAL ptr [esi]).pTextureFilename ;		
		.if dword ptr [esi] != 0
			mov esi,dword ptr [esi]	;[esi] tiger.bmp
			;--------------------------------------
			mov edi,sizeof DWORD
			imul edi,index
			add edi,g_pMeshTextures
			invoke lstrlen,esi
			.if eax > 0
				;the .x is text file,changed "tiger.bmp" in "tiger_bmp"
				;and like that have a named resource,bye bye "couldn't find.."
				invoke D3DXCreateTextureFromResource,ppvIDirect3DDevice9,NULL,\
						esi,edi;   edi= addr ppvIDirect3DTexture9 ;
				.if eax != D3D_OK
	                		;If texture is not in current folder, try parent folder
					;ajouter le prfixe "..\\" au nom de fichier dans un tampon	
					invoke MessageBox,NULL,SADR("Resource not found"),SADR("tiger_bmp"),MB_OK
					jmp FindeInitVB
				.endif		
				
			.endif
		.endif
		
		
		inc index			
		dec CNT
	.endw	
	mov retour,1
	;
	;reste les deux tampons mmoires et les interfaces
	;le pointeur en data com est dfinitivement perdu
 FindeInitVB:
	.if ppvID3DXBuffer != 0
		ID3DXBuffer	Release
		mov ppvID3DXBuffer,0
	.endif
         mov eax,retour
         ret
InitVB endp

SetupMatrices PROC uses edi esi
	;incertitude,alignement en pile ?
		Local iTime:DWORD,time:DWORD
		align 16		;obligatoire
		local matWorld:D3DXMATRIXA16 
		align 16
		local matView:D3DXMATRIXA16
		align 16
		local matProj:D3DXMATRIXA16		
		LOcal vEyePt	:STD3DXVECTOR3
		Local vLookatPt	:STD3DXVECTOR3
		Local vUpVec	:STD3DXVECTOR3
		Local PasseQword	:REAL8
		Local Angle		:DWORD
		Local  retour:DWORD
		ZEROLOCALES retour
		; For our world matrix, we will just leave it as the identity				
		;fonction du temps,diffrente de matrices
		invoke timeGetTime ;temps en ms
		mov time,eax
		;-------- calcul de l'angle de rotation en radians --------------
		fild	dword ptr time	;charge le nombre de millimes de tour
		fdiv	QWORD PTR mille	;on retrouve une fraction de tour en radians < 6,28
		fstp	DWORD PTR Angle		;charg dans l'angle
		invoke D3DXMatrixRotationY,addr matWorld,Angle		
		IDirect3DDevice9 SetTransform, D3DTS_WORLD,addr matWorld
		;-----------------------------------------------------
		
		; Set up our view matrix. A view matrix can be defined given an eye point,
		; a point to lookat, and a direction for which way is up. Here, we set the
		;eye five units back along the z-axis and up three units, look at the
		;origin, and define "up" to be in the y-direction.
		LOADD3DXVECTOR3 vEyePt,0.0f, 3.0f,-5.0f
		LOADD3DXVECTOR3 vLookatPt,0.0f, 0.0f, 0.0f
		LOADD3DXVECTOR3 vUpVec,0.0f,1.0f, 0.0f
		invoke D3DXMatrixLookAtLH,addr matView,addr vEyePt,addr vLookatPt,addr vUpVec 
		IDirect3DDevice9 SetTransform,D3DTS_VIEW,addr matView
		
		;For the projection matrix, we set up a perspective transform (which
		;transforms geometry from 3D view space to 2D viewport space, with
		; a perspective divide making objects smaller in the distance). To build
		;a perpsective transform, we need the field of view (1/4 pi is common),
		;the aspect ratio, and the near and far clipping planes (which define at
		;what distances geometry should be no longer be rendered).
		invoke D3DXMatrixPerspectiveFovLH,addr matProj,UnQuartPi,FR4(1.0f),FR4(1.0f),FR4(100.0f)
		IDirect3DDevice9 SetTransform,D3DTS_PROJECTION,addr matProj
		
         mov retour,1
  ; For our world matrix, we will just rotate the object about the y-axis.

 FindeSetupMatrices:
         mov eax,retour
         ret

SetupMatrices endp

;################################################################
SetupMeshes PROC 
	Local index:DWORD,CNT
	Local retour:DWORD
        ; Meshes are divided into subsets, one for each material. Render them in
        ; a loop
	PuPo CNT,dwNumMaterials
	mov index,0
	.while(CNT)
            ; Set the material and texture for this subset
		mov ecx,sizeof D3DMATERIAL9
		imul ecx,index
		add ecx,g_pMeshMaterials
            IDirect3DDevice9 SetMaterial,ecx
		mov ecx,sizeof DWORD
		imul ecx,index
		add ecx,g_pMeshTextures
		mov ecx,[ecx]
            IDirect3DDevice9 SetTexture,0,ecx
        
            ;Draw the mesh subset
            ID3DXMesh DrawSubset,index
		inc index
		dec CNT
	.endw
	
 FindeSetupMeshes:
         mov eax,retour
         ret
SetupMeshes endp

;################################################################




;################################################################
Render PROC 
			Local Zreel:DWORD
			Local  retour:DWORD
         mov retour,1
	.if ppvIDirect3DDevice9 != 0
		;arriere plan bleu
		invoke RGBcolorARGB,00FF0000h,NULL		;bleu
		mov ecx,eax
		;erreur traduction D3DCLEAR_ZBUFFER
		;invalid_call avec +D3DCLEAR_ZBUFFER  oubli de  d3dpp.EnableAutoDepthStencil = TRUE;
		IDirect3DDevice9 Clear,0, NULL,D3DCLEAR_TARGET+D3DCLEAR_ZBUFFER,ecx,FR4(1.0f), 0 
		IDirect3DDevice9 BeginScene
		.if eax != D3D_OK	
			call MessageNotD3D_OK
		.else
		
        		; Setup the world, view, and projection matrices
			IF Exceptions EQ 1
				XPEXCEPT SetupMatrices
				XPEXCEPT SetupMeshes
			ELSE
        			invoke SetupMatrices
				invoke SetupMeshes
			ENDIF
			IDirect3DDevice9 EndScene
		.endif
		IDirect3DDevice9 Present,NULL, NULL, NULL, NULL
	.endif
	
 FindeRender:
         mov eax,retour
         ret
MessageNotD3D_OK:	
	pop eax    ;toujours garder l'quilibre de la pile en push pop
	invoke MessageBox,NULL,SADR("Render not D3D_OK") ,SADR("Erreur Quitte"),MB_OK
	jmp FindeRender ;retour = 0
retn
Render endp
;la fonction ne fait qu'une seule chose
;plac les rels dans la structure
;reoit l'adresse de la structure par ecx
;_this,est un terme,en principe,reserv au pointeur com(introduisant le ppv)
;ii,aucune manipulation de pointeur,tout est direct
;plus confus ,ce n'est pas possible
;################################################################
D3DXVECTOR3 PROC  fx:REAL4,  fy:REAL4,  fz:REAL4 
		local _this:DWORD
		Local  retour:DWORD
      mov retour,1
	mov	_this, ecx
;il faut comprendre structure.x = fx	structure = STD3DXVECTOR3
; x = fx;
	mov	eax, _this
	fld	DWORD PTR fx
	fstp	DWORD PTR [eax]		
;  y = fy;
	mov	eax, _this
	fld	DWORD PTR fy
	fstp	DWORD PTR [eax+4]
; z = fz;
	mov	eax, _this
	fld	DWORD PTR fz
	fstp	DWORD PTR [eax+8]
;

	PuPo retour,_this
 FindeD3DXVECTOR3:
         mov eax,retour
         ret
D3DXVECTOR3 endp

D3DXMatrixIdentity PROC pOut:DWORD	; D3DXMatrixIdentity, COMDAT
;zro gnral fait par zro locales
; 1928 : 
; 1929 :     pOut->m[0][0] = pOut->m[1][1] = pOut->m[2][2] = pOut->m[3][3] = 1.0f;

	mov	eax,pOut
	fld1
	fstp	DWORD PTR [eax+60]
	mov	ecx,pOut
	fld1
	fstp	DWORD PTR [ecx+40]
	mov	edx, DWORD PTR pOut
	fld1
	fstp	DWORD PTR [edx+20]
	mov	eax,pOut
	fld1
	fstp	DWORD PTR [eax]

	mov	eax, DWORD PTR pOut

	ret
D3DXMatrixIdentity ENDP	; D3DXMatrixIdentity



;################################################################
RGBcolorARGB PROC  RGB:DWORD,  ARGB:DWORD 
			Local  retour:DWORD
		;plein les mirettes
		;Red Green Blue RGB donne aprs decalage 00BGR
		;ARGB donne aprs dcalage ARGB
		;ARGB color macro pour extraction,extraire sparment du RGB
		;0000FFFFh jaune RGB B=00 G=FF R=FF     ARGB 0FFFFFF00h
		;#define RGB(r, g ,b)  (r | g << 8 | b << 16 )
		;#define D3DCOLOR_ARGB(a,r,g,b)((a)&0xff)<<24)|(r)<<16)|g <<8|(b)		
			
      mov retour,0
	.if ARGB == 0	
		;on veut une couleur ARGB
		and edx,0FF000000h	;
		mov eax,RGB
		and eax,0FF0000h  ;bleu
		shr eax,16		
		or edx,eax
		mov eax,RGB
		and eax,0FF00h	;green vert
		or edx,eax
		mov eax,RGB
		and eax,0FFh	;rouge
		shl eax,16
		or edx,eax
		mov retour,edx				
	.else		
		;on veut une couleur RGB
		and edx,0
		mov eax,ARGB
		and eax,0FF0000h  ;rouge
		shr eax,16		
		or edx,eax
		mov eax,ARGB
		and eax,0FF00h	;green vert
		or edx,eax
		mov eax,ARGB
		and eax,0FFh	;Bleu		
		shl eax,16
		or edx,eax
		mov retour,edx						
		
	.endif

 FindeRGBcolorARGB:
         mov eax,retour
         ret
RGBcolorARGB endp

;################################################################
Sinus PROC  AngleRadian:DWORD 
			Local   result:DWORD
			FINIT
			fld	dword PTR AngleRadian	;s+1
			fsin
			fstp	DWORD PTR result		;s-1
			mov eax,result
         ret
Sinus endp

;################################################################

Cosinus PROC  AngleRadian:DWORD 
		Local result:DWORD
			FINIT
			fld	DWORD PTR AngleRadian
			fcos
			fstp	DWORD PTR result
			mov eax,result
         ret
Cosinus endp

;################################################################





;################################################################

Cleanup PROC      ;relache les interfaces
			Local CNT:DWORD,index
			Local  retour:DWORD
         mov retour,1
	.if g_pMeshTextures != 0
		PuPo CNT,dwNumMaterials
		mov index,0
		.while(CNT)
			mov ecx,sizeof DWORD
			imul ecx,index
			add ecx,g_pMeshTextures
			PuPo ppvIDirect3DTexture9,[ecx]
			IDirect3DTexture9 Release
			inc index
			dec CNT
		.endw
		mov ppvIDirect3DTexture9,0
		;effacement de la mmoire avec Imalloc,plus tard
	.endif			
	.if g_pMeshMaterials != 0
		IMalloc Free,g_pMeshMaterials
		mov g_pMeshMaterials,0
	.endif
	.if g_pMeshTextures != 0
		IMalloc Free,g_pMeshTextures
		mov g_pMeshTextures,0
	.endif	
	.if ppvIMalloc != 0
		IMalloc Release
		mov ppvIMalloc,0
	.endif
	;------------------ interface -----------------
	
	.if ppvIDirect3DVertexBuffer9 != 0
		IDirect3DVertexBuffer9 Release
		mov ppvIDirect3DVertexBuffer9,0
	.endif

	.if ppvIDirect3DDevice9 != 0
		IDirect3DDevice9 Release
		mov ppvIDirect3DDevice9,0
	.endif
	.if ppvIDirect3D9 != 0
		IDirect3D9 Release
		mov ppvIDirect3D9,0
	.endif

 FindeCleanup:
         mov eax,retour
         ret
Cleanup endp





